var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
import { ReactWidget } from '@jupyterlab/apputils';
import { defaultSanitizer } from '@jupyterlab/apputils';
import React, { useRef, useEffect } from 'react';
import { CodeEditor } from '@jupyterlab/codeeditor';
import { CodeMirrorEditor } from '@jupyterlab/codemirror';
import { markdownRendererFactory, MimeModel } from '@jupyterlab/rendermime';
import * as marked from '@jupyterlab/markedparser-extension';
import { nullTranslator } from '@jupyterlab/translation';
function createModel(mimeType, source, trusted = false) {
    const data = {};
    data[mimeType] = source;
    return new MimeModel({ data, trusted });
}
const sanitizer = defaultSanitizer;
const defaultOptions = {
    sanitizer,
    linkHandler: null,
    resolver: null
};
function CodeViewer(props) {
    const ref = useRef(null);
    useEffect(() => {
        if (ref.current) {
            async function callback() {
                var e_1, _a;
                var _b, _c;
                try {
                    for (var _d = __asyncValues(props.file.cells), _e; _e = await _d.next(), !_e.done;) {
                        const cell = _e.value;
                        if (cell.cell_type === 'code') {
                            const wrapper = document.createElement('div');
                            wrapper.className =
                                'lm-Widget p-Widget jp-Cell jp-CodeCell jp-mod-noOutputs jp-Notebook-cell';
                            const host = document.createElement('div');
                            host.className =
                                'lm-Widget p-Widget jp-CodeMirrorEditor jp-Editor jp-InputArea-editor';
                            const model = new CodeEditor.Model({
                                value: cell.source,
                                mimeType: 'ipython'
                            });
                            new CodeMirrorEditor({ host, model });
                            wrapper.appendChild(host);
                            (_b = ref.current) === null || _b === void 0 ? void 0 : _b.appendChild(wrapper);
                        }
                        else {
                            const markdownParser = marked.default.activate(props.app);
                            const f = markdownRendererFactory;
                            const source = cell.source;
                            const mimeType = 'text/markdown';
                            const model = createModel(mimeType, source);
                            const wrapperDiv = document.createElement('div');
                            wrapperDiv.className =
                                'lm-Widget p-Widget jp-InputArea jp-Cell-inputArea';
                            const w = f.createRenderer(Object.assign(Object.assign({ mimeType }, defaultOptions), { markdownParser }));
                            await w.renderModel(model);
                            w.node.className =
                                'lm-Widget p-Widget jp-RenderedHTMLCommon jp-RenderedMarkdown jp-MarkdownOutput';
                            wrapperDiv.appendChild(w.node);
                            (_c = ref.current) === null || _c === void 0 ? void 0 : _c.appendChild(wrapperDiv);
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_e && !_e.done && (_a = _d.return)) await _a.call(_d);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            callback();
        }
    }, [ref]);
    return (React.createElement("div", { className: "lm-Widget p-Widget jp-MainAreaWidget jp-NotebookPanel jp-Document jp-Activity lm-DockPanel-widget p-DockPanel-widget", style: { overflow: 'scroll' } },
        React.createElement("div", { className: "mutable-ai-content-header" },
            React.createElement("div", null,
                React.createElement("span", { onClick: props.handlers.onAcceptChanges }, props.trans.__('Accept')),
                ' or ',
                React.createElement("span", { onClick: props.handlers.onDeclineChanges }, props.trans.__('Decline')),
                props.trans.__(' changes?'))),
        React.createElement("div", { ref: ref, className: "lm-Widget p-Widget jp-Notebook jp-mod-scrollPastEnd jp-NotebookPanel-notebook jp-mod-commandMode" })));
}
export class CodeViewerWidget extends ReactWidget {
    constructor(close, file, app, handlers, translator) {
        super();
        // This is used to close the shell.
        this.closeShell = close;
        this.file = file;
        this.app = app;
        this.handlers = handlers;
        this.trans = (translator || nullTranslator).load('jupyterlab');
    }
    render() {
        // This is the Code Viewer component passed to the widget.
        return (React.createElement(CodeViewer, { close: () => this.closeShell(), file: this.file, app: this.app, handlers: this.handlers, trans: this.trans }));
    }
}
